#!/usr/bin/perl -w
#
# Caller-ID-logger, by jwz (19-Jan-97)
# Modified: 05-Mar-2000 by Suraj K. Rai (surajrai@yahoo.com)
#
# The original script is available at JWZ's website http://www.jwz.org
# I basically hacked his script to suit my needs...which is to be able to 
# log the calls into a file with the callerid info.  
#
# I have another script which parses the log file and makes a nice web page 
# so that I can check my call-logs via a browser.
# 
# Of course if you want to go really wild...you could do what Jamie does, i.e.
# to ring softly when screensaver is off, display the cid to his laptop etc. etc.
# Read more about it in his homepage.
#
# The modem used is a US Robotics Courier modem so all the AT commands are
# meant for the USR modem.  It most probably won't work for other modems.
# I am using SDL version 7.3.14 to get the CID info ... you might want to upgrade
# the modem software if you can't get CID to work for you.
#
# The USR modem spits out CID data in the following format:
#
#  DATE = 0302
#  TIME = 1030
#  NMBR = 8005551212
#  NAME = Joe Schmoe

##############################################################################
#
# Some variables you might want to set...

# Set this to the device that your modem is attached to.
# NOTE LINUX USERS: since we are using cu to read from /dev/ttyS0...make sure
# that /dev/ttyS0 is readable/writable by the group, user uucp
#
$modem_device = "/dev/ttyS0";

# If you want each call to be logged to a file as well, name it here.
$logfile      = "/usr/local/etc/cid-log";

# For verbosity...
$debug        = 1;

# commands (and their expected responses) used to initialize the modem.
#
@modem_init   = ( "AT",     "OK", # ping
                  "ATZ",    "OK", # reset
                  "ATE0",   "OK", # don't echo commands
                  "ATM0",   "OK", # turn off speaker
                  "ATS0=0", "OK", # turn off auto-answer...
                  "AT#CID=1",   "OK",         # turn on caller ID
                );

# for diagnostics: if the modem ever asynchronously prints something that
# doesn't match this, we issue a warning.
#
$expected_responses = "^NMBR"      . "|" .
    "^NAME"        . "|" .
    "^DATE"         . "|" .
    "^TIME"         . "|" .
    "^RING"         . "|" .
    "^NO CARRIER";

# Global vars
$name      = "";
$number    = "";

##############################################################################
#
# Talking to the serial port...
#
#

if ( $ debug ) {
    use diagnostics;
}


sub DEBUG {
    if ( $debug ) {
        my ($str) = @_;
        my $d = localtime;
        print STDERR "$d:   $str\n";
    }
}

sub open_modem {
    use IPC::Open2;
    
    my $cu_pid = open2( \*MODEM_IN, \*MODEM_OUT,
                        "cu -l$modem_device -s2400");

    # don't know why the following does not work with linux
    # if we uncomment the following the io seems to get messed up
    # and nothing works from here on.
#     $_ = <MODEM_IN>;
#     chop;
#     if ( !m/^Connected/ ) {
#         DEBUG "$0: cu printed `$_' instead of `Connected'";
#     }
}

sub read_line {
    $_ = <MODEM_IN>;
    $_ || die("got eof on modem");
    s/[\r\n]+$//;
    if ( $_ eq "" ) {
        $_ = <MODEM_IN>;
        $_ || die("got eof on modem");
        s/[\r\n]+$//;
    }
    return $_;
}

sub command {
    my ( $command, $expected_response) = @_;
    
    if ( $debug ) {
        DEBUG "sending: $command";
    }
    
    print MODEM_OUT "$command\r\n";
    my $line = read_line();
    
    if ( $line eq $command ) {
        if ( $debug ) {
            DEBUG "    got echo: reading next line too...";
        }
        $line = read_line();
    }
    
    if ( $line ne $expected_response ) {
        DEBUG "    got: $line ; expected: $expected_response";
    } elsif ( $debug ) {
        DEBUG "    got: $line";
    }
}

sub init_modem {
    open_modem;
    
    my $len = $#modem_init + 1;
    my $i;
    for ($i = 0; $i < $len; $i += 2) {
        command($modem_init[$i], $modem_init[$i+1]);
    }
}

sub handle_async_line {
    local ( $_ ) = @_;
    
    if (!m/$expected_responses/) {
        DEBUG "modem turd:   $_";
        
    } elsif (m/(NMBR)|(NAME)/) { # ignore DATE and TIME
        if ( $debug ) {
            DEBUG "cid_line: $_";
        }
        handle_cid_line($_);
    } elsif ( $debug ) {
        if ( $_ eq '' ) {
            DEBUG "ignored: blank line";
        } else {
            DEBUG "ignored: $_";
        }
    }
}

sub handle_cid_line {
    my($line) = @_;
    
    my $date = time; # use date/time in epoch format so easier to do date operations
    
    $_ = $line;

    if( m/NMBR/ ) {
        
        m/(\d+)/;
        $_ = $1;

        my $area = 0;
        my $exchange = 0;
        my $suffix = 0;
        ( $area, $exchange, $suffix ) =
            m/^([0-9][0-9][0-9])([0-9][0-9][0-9])([0-9][0-9][0-9][0-9]+)/;
        
        $number = "($area) $exchange-$suffix";
    } else { # has to be the name
        m/(^NAME = )(.*)/;
        $name = $2;
        $name =~ s/[ \r\n]*$//;
        $name =~ s/,/ /;

        # Log the call...as a comma separated directory
        #
        if ( $logfile ) {
            if (open(LOG, ">>$logfile")) {
                print LOG "$date,$name,$number\n";
                close LOG;
            } else {
                DEBUG "error opening $logfile: $!";
            }
        }

        # clean up for next call
        $name = "";
        $number = "";
    }
}

##############################################################################
#
# hey ho.  let's go.
#
sub main {
    init_modem();
    while (1) {
        handle_async_line(read_line());
    }
    exit (1);
}

main();
